use tokio::{io::AsyncReadExt, net::TcpListener};
use crate::message_read::MessageReader;

pub struct Server {
    pub host: String,
    pub port: u16,
}

impl Server {
    pub fn new(host: impl Into<String>, port: u16) -> Self {
        Server {
            host: host.into(),
            port,
        }
    }

    pub async fn run(&mut self) -> anyhow::Result<()> {
        let listener = TcpListener::bind(format!(
            "{}:{}",
            self.host,
            self.port
        )).await?;

        println!("Server is running on {}:{}", self.host, self.port);

        loop {
            let (mut socket, addr) = listener.accept().await?;
            println!("Connection received from {}", addr);

            tokio::task::spawn(async move {
                let mut message_reader = MessageReader::new();
                'handler: loop {
                    let mut buffer = vec![0; 256];
                    let bytes_read = socket.read(&mut buffer).await?;

                    let messages = message_reader.read(&buffer[..bytes_read])?;

                    for message in messages {
                        if message.content == "EXIT" {
                            println!("Connection closed by the client");
                            break 'handler;
                        }

                        println!("Message:\n{:?}", message);
                    }
                }

                Ok::<(), anyhow::Error>(())
            });
        }

        Ok(())
    }
}
